﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace UltimatumGame
{
    public enum cerationType
    {
        Uniform,
        Random,
        Evil_LH,
        Good_HH,
        Naive_HL,
        Greedy_LL,
        LowOffer,
        HighOffer,
        LowAccept,
        HighAccept
    }
    
    class GamePlayer
    {
        public long score
        {
            get;
            private set;
        }
        public double offerPercentage
        {
            get;
            private set;
        }

        public int roundsPlayed
        {
            get;
            private set;
        }

        public double acceptancePercentage
        {
            get;
            private set;
        }
        
        public void receiveReward(int reward)
        {
            this.score += reward;
        }

        public bool willAccept(int offer, int totalAmount)
        {
            roundsPlayed++;
            return totalAmount * acceptancePercentage <= offer;
        }

        public int makeOffer(int totalAmount)
        {
            roundsPlayed++;
            return (int)(totalAmount * offerPercentage);
        }

        public GamePlayer(double offer, double acceptance)
        {
            this.acceptancePercentage = acceptance;
            this.offerPercentage = offer;
            this.score = 0;
            this.roundsPlayed = 0;
            if (acceptance > 1 || offer > 1)
            {
                throw new Exception();
            }
        }

        public void reset()
        {
            this.score = 0;
            this.roundsPlayed = 0;
        }
    }

    class GameMaster
    {
        public List<GamePlayer> players;
        static Random r = new Random();
        public long totalStepsTaken = 0;
        public bool evolution = false;

        public int tournamentSize = 3;
        public double mutationChance = 0.05, mutationRange = 0.1;
        public int evolutionPhase = 1000;

        public int amountToPlay
        {
            get;
            set;
        }

        public void playOneTurn()
        {
            totalStepsTaken++;
            if (evolution && totalStepsTaken % evolutionPhase == evolutionPhase - 1)
                evolve();
            int firstPlayerIndex = r.Next(players.Count()),
                secondPlayerIndex = r.Next(players.Count());
            while (firstPlayerIndex == secondPlayerIndex)
                secondPlayerIndex = r.Next(players.Count());

            int offer = players[firstPlayerIndex].makeOffer(amountToPlay);
            if (players[secondPlayerIndex].willAccept(offer, amountToPlay))
            {
                players[firstPlayerIndex].receiveReward(amountToPlay - offer);
                players[secondPlayerIndex].receiveReward(offer);
            }
        }

        private void evolve()
        {
            //select the weakest player (tournament selection)
            int tournamentSize = 3;
            double mutationChance = 0.05, mutationRange = 0.1;
            List<GamePlayer> candidates = new List<GamePlayer>(tournamentSize);
            for (int i = 0; i < tournamentSize; i++)
            {
                candidates.Add(players[r.Next(players.Count)]);
            }
            GamePlayer weakest = candidates[0];
            double weakestScore = weakest.roundsPlayed == 0 ? weakest.score : weakest.score / weakest.roundsPlayed;
            for (int i = 1; i < candidates.Count; i++)
            {
                if (candidates[i].roundsPlayed > 0 && candidates[i].score / candidates[i].roundsPlayed < weakestScore)
                {
                    weakest = candidates[i];
                    weakestScore = candidates[i].score / candidates[i].roundsPlayed;
                }
            }
                      
            //select parrents (tournament selection)
            List<GamePlayer> candidates1 = new List<GamePlayer>(tournamentSize), candidates2 = new List<GamePlayer>(tournamentSize);
            for (int i = 0; i < tournamentSize; i++)
            {
                candidates1.Add(players[r.Next(players.Count)]);
                candidates2.Add(players[r.Next(players.Count)]);
            }
            GamePlayer parrent1 = candidates1[0], parrent2 = candidates2[0];
            double score1 = parrent1.roundsPlayed == 0 ? parrent1.score : parrent1.score / parrent1.roundsPlayed,
                score2 = parrent2.roundsPlayed == 0 ? parrent2.score : parrent2.score / parrent2.roundsPlayed;
            for (int i = 1; i < candidates1.Count; i++)
            {
                if (candidates1[i].roundsPlayed > 0 && candidates1[i].score / candidates1[i].roundsPlayed > score1)
                {
                    parrent1 = candidates1[i];
                    score1 = candidates1[i].score / candidates1[i].roundsPlayed;
                }
                if (candidates2[i].roundsPlayed > 0 && candidates2[i].score / candidates2[i].roundsPlayed > score2)
                {
                    parrent2 = candidates2[i];
                    score2 = candidates2[i].score / candidates2[i].roundsPlayed;
                }
            }

            //create offspring and replece weakest agent by the offstring
            double offspringAcc = (parrent1.acceptancePercentage + parrent2.acceptancePercentage) / 2,
            offspringOff = (parrent1.offerPercentage + parrent2.offerPercentage) / 2;
            if (r.NextDouble() < mutationChance)
            {
                offspringAcc += r.NextDouble() * mutationRange - mutationRange / 2;
                offspringOff += r.NextDouble() * mutationRange - mutationRange / 2;
            }
            if (offspringAcc < 0)
                offspringAcc = 0;
            if (offspringAcc > 1)
                offspringAcc = 1;
            if (offspringOff < 0)
                offspringOff = 0;
            if (offspringOff > 1)
                offspringOff = 1;

            GamePlayer offspring = new GamePlayer(offspringOff, offspringAcc);
            players.Remove(weakest);
            players.Add(offspring);
        }

        public void addPlayer(GamePlayer p)
        {
            this.players.Add(p);
        }

        public GameMaster()
        {
            this.players = new List<GamePlayer>();
            this.amountToPlay = 1000;
        }

        public void reset()
        {
            foreach (var item in players)
            {
                item.reset();
            }
        }
    }
}
