﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace UltimatumGame
{
    public partial class Form1 : Form
    {
        private GameMaster gm;
        private Drawer scoreDrawer, countsDrawer;

        public Form1()
        {
            InitializeComponent();
            comboBox1.SelectedIndex = 0;
        }

        private void initialize(int granularity, cerationType type)
        {
            Random r = new Random();
            gm = new GameMaster();
            scoreDrawer = new Drawer(pictureBox1, gm, granularity + 1);
            countsDrawer = new CountsDrawer(pictureBox2, gm, granularity + 1);

            switch (type)
            {
                case cerationType.Uniform:
                    for (int i = 0; i < granularity + 1; i++)
                    {
                        for (int j = 0; j < granularity + 1; j++)
                        {
                            GamePlayer p = new GamePlayer((double)(i) / granularity, (double)(j) / granularity);
                            gm.addPlayer(p);
                        }
                    }
                    break;
                case cerationType.Random:
                    for (int i = 0; i < granularity + 1; i++)
                    {
                        for (int j = 0; j < granularity + 1; j++)
                        {
                            GamePlayer p = new GamePlayer(r.NextDouble(), r.NextDouble());
                            gm.addPlayer(p);
                        }
                    }
                    break;
                case cerationType.Evil_LH:
                    for (int i = 0; i < granularity + 1; i++)
                    {
                        for (int j = 0; j < granularity + 1; j++)
                        {
                            if (i > granularity * 0.9)
                            {
                                GamePlayer p = new GamePlayer(r.NextDouble(), r.NextDouble());
                                gm.addPlayer(p);
                            }
                            else
                            {
                                GamePlayer p = new GamePlayer(0.25 + (r.NextDouble() / 2) - 0.25, 0.75 + (r.NextDouble() / 2) - 0.25);
                                gm.addPlayer(p);
                            }
                        }
                    }
                    break;
                case cerationType.Good_HH:
                    for (int i = 0; i < granularity + 1; i++)
                    {
                        for (int j = 0; j < granularity + 1; j++)
                        {
                            if (i > granularity * 0.9)
                            {
                                GamePlayer p = new GamePlayer(r.NextDouble(), r.NextDouble());
                                gm.addPlayer(p);
                            }
                            else
                            {
                                GamePlayer p = new GamePlayer(0.75 + (r.NextDouble() / 2) - 0.25, 0.75 + (r.NextDouble() / 2) - 0.25);
                                gm.addPlayer(p);
                            }
                        }
                    }
                    break;
                case cerationType.Greedy_LL:
                    for (int i = 0; i < granularity + 1; i++)
                    {
                        for (int j = 0; j < granularity + 1; j++)
                        {
                            if (i > granularity * 0.9)
                            {
                                GamePlayer p = new GamePlayer(r.NextDouble(), r.NextDouble());
                                gm.addPlayer(p);
                            }
                            else
                            {
                                GamePlayer p = new GamePlayer(0.25 + (r.NextDouble() / 2) - 0.25, 0.25 + (r.NextDouble() / 2) - 0.25);
                                gm.addPlayer(p);
                            }
                        }
                    }
                    break;
                case cerationType.Naive_HL:
                    for (int i = 0; i < granularity + 1; i++)
                    {
                        for (int j = 0; j < granularity + 1; j++)
                        {
                            if (i > granularity * 0.9)
                            {
                                GamePlayer p = new GamePlayer(r.NextDouble(), r.NextDouble());
                                gm.addPlayer(p);
                            }
                            else
                            {
                                GamePlayer p = new GamePlayer(0.75 + (r.NextDouble() / 2) - 0.25, 0.25 + (r.NextDouble() / 2) - 0.25);
                                gm.addPlayer(p);
                            }
                        }
                    }
                    break;
                case cerationType.HighAccept:
                    for (int i = 0; i < granularity + 1; i++)
                    {
                        for (int j = 0; j < granularity + 1; j++)
                        {
                            if (i > granularity * 0.9)
                            {
                                GamePlayer p = new GamePlayer(r.NextDouble(), r.NextDouble());
                                gm.addPlayer(p);
                            }
                            else
                            {
                                GamePlayer p = new GamePlayer(r.NextDouble(), 0.9 + (r.NextDouble() / 5) - 0.1);
                                gm.addPlayer(p);
                            }
                        }
                    }
                    break;
                case cerationType.LowAccept:             
                    for (int i = 0; i < granularity + 1; i++)
                    {
                        for (int j = 0; j < granularity + 1; j++)
                        {
                            if (i > granularity * 0.9)
                            {
                                GamePlayer p = new GamePlayer(r.NextDouble(), r.NextDouble());
                                gm.addPlayer(p);
                            }
                            else
                            {
                                GamePlayer p = new GamePlayer(r.NextDouble(), 0.1 + (r.NextDouble() / 5) - 0.1);
                                gm.addPlayer(p);
                            }
                        }
                    }
                    break;
                case cerationType.LowOffer:
                    for (int i = 0; i < granularity + 1; i++)
                    {
                        for (int j = 0; j < granularity + 1; j++)
                        {
                            if (i > granularity * 0.9)
                            {
                                GamePlayer p = new GamePlayer(r.NextDouble(), r.NextDouble());
                                gm.addPlayer(p);
                            }
                            else
                            {
                                GamePlayer p = new GamePlayer(0.1 + (r.NextDouble() / 5) - 0.1, r.NextDouble());
                                gm.addPlayer(p);
                            }
                        }
                    }
                    break;
                case cerationType.HighOffer:
                    for (int i = 0; i < granularity + 1; i++)
                    {
                        for (int j = 0; j < granularity + 1; j++)
                        {
                            if (i > granularity * 0.9)
                            {
                                GamePlayer p = new GamePlayer(r.NextDouble(), r.NextDouble());
                                gm.addPlayer(p);
                            }
                            else
                            {
                                GamePlayer p = new GamePlayer(0.9 + (r.NextDouble() / 5) - 0.1, r.NextDouble());
                                gm.addPlayer(p);
                            }
                        }
                    }
                    break;
            }
            scoreDrawer.draw(); 
            countsDrawer.draw();
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            StepsCount_label.Text = getSteps(trackBar1.Value) + " steps";
            
        }

        private int getSteps(int trackBarValue)
        {
            /*
            switch (trackBarValue)
            {
                case 0: return 1;
                case 1: return 5;
                case 2: return 10;
                case 3: return 100;
                case 4: return 1000;
                case 5: return 10000;
                case 6: return 100000;
                case 7: return 1000000;
            }
            return 0;
             */
            return (int)Math.Pow(10, trackBarValue);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (gm == null)
            {
                MessageBox.Show("First create a scenario using the \"create\" button.", "Error");
                return;
            }
            int steps = getSteps(trackBar1.Value);
            gm.evolution = checkBox1.Checked;
            for (int i = 0; i < steps; i++)
            {
                gm.playOneTurn();
            }
            scoreDrawer.draw();
            countsDrawer.draw();
        }

        private void Graph3Dbutton_Click(object sender, EventArgs e)
        {
            try
            {
                scoreDrawer.create3DGraph("file.html");
                //MessageBox.Show("Graph created in file.html");
                System.Diagnostics.Process.Start(@"..\Graph\file.html");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Nothing to show. First create a scenario using the \"create\" button.", "Error");
            }
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            int granularity = int.Parse(textBox1.Text);
            initialize(granularity, (UltimatumGame.cerationType)comboBox1.SelectedIndex);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                pictureBox1.Image.Save(@"..\Pics\picture1.png", System.Drawing.Imaging.ImageFormat.Png);
                MessageBox.Show("Saved as picture1.png");
            }
            catch (Exception ex)
            {
                MessageBox.Show("No picture to save. First create a scenario using the \"create\" button.", "Error");
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            try
            {
                pictureBox2.Image.Save(@"..\Pics\picture2.png", System.Drawing.Imaging.ImageFormat.Png);
                MessageBox.Show("Saved as picture2.png");
            }
            catch (Exception ex)
            {
                MessageBox.Show("No picture to save. First create a scenario using the \"create\" button.", "Error");
            }
        }
    }
}
